# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""simplify state transition pattern format

Revision ID: 2c098eceee99
Revises: adb4a976c9f8
Create Date: 2021-10-11 10:30:37.794423

"""
from alembic import op

# revision identifiers, used by Alembic.
revision = "2c098eceee99"
down_revision = "adb4a976c9f8"
branch_labels = None
depends_on = None


def upgrade():
    op.execute(
        """
        DELETE FROM capability_state_actions CASCADE;
        DELETE FROM capability_state_transitions CASCADE;

        INSERT INTO capability_state_transitions (capability_name, from_state, pattern, to_state)
        SELECT capability_name, 'Start', 'event_type == capability-submitted', 'Executing' FROM capabilities
        WHERE capability_steps ~ E'^prepare-and-run-workflow (.*)\nawait-workflow$';
        INSERT INTO capability_state_transitions (capability_name, from_state, pattern, to_state)
        SELECT capability_name, 'Executing', 'event_type == workflow-complete', 'Complete' FROM capabilities
        WHERE capability_steps ~ E'^prepare-and-run-workflow (.*)\nawait-workflow$';
        INSERT INTO capability_state_transitions (capability_name, from_state, pattern, to_state)
        SELECT capability_name, 'Executing', 'event_type == workflow-failed', 'Failed' FROM capabilities
        WHERE capability_steps ~ E'^prepare-and-run-workflow (.*)\nawait-workflow$';

        INSERT INTO capability_state_actions (transition_id, action_type, arguments)
        SELECT transition_id, 'QueueWorkflow', (regexp_match(capability_steps, E'^prepare-and-run-workflow (.*)\nawait-workflow$'))[1] as arguments
        FROM capability_state_transitions cst
        JOIN capabilities c on cst.capability_name = c.capability_name
        WHERE cst.from_state = 'Start' and cst.to_state = 'Executing';
        INSERT INTO capability_state_actions (transition_id, action_type, arguments)
        SELECT transition_id, 'SendNotification', 'workflow-complete'
        FROM capability_state_transitions cst
        WHERE cst.from_state = 'Executing' and cst.to_state = 'Complete';
        INSERT INTO capability_state_actions (transition_id, action_type, arguments)
        SELECT transition_id, 'SendNotification', 'workflow-failed'
        FROM capability_state_transitions cst
        WHERE cst.from_state = 'Executing' and cst.to_state = 'Failed';
        """
    )


def downgrade():
    op.execute(
        """
        DELETE FROM capability_state_actions CASCADE;
        DELETE FROM capability_state_transitions CASCADE;

        INSERT INTO capability_state_transitions (capability_name, from_state, pattern, to_state)
        SELECT capability_name, 'Start', 'exists($ ? (@.event_type == "capability-submitted"))', 'Executing' FROM capabilities
        WHERE capability_steps ~ E'^prepare-and-run-workflow (.*)\nawait-workflow$';
        INSERT INTO capability_state_transitions (capability_name, from_state, pattern, to_state)
        SELECT capability_name, 'Executing', 'exists($ ? (@.event_type == "workflow-complete"))', 'Complete' FROM capabilities
        WHERE capability_steps ~ E'^prepare-and-run-workflow (.*)\nawait-workflow$';
        INSERT INTO capability_state_transitions (capability_name, from_state, pattern, to_state)
        SELECT capability_name, 'Executing', 'exists($ ? (@.event_type == "workflow-failed"))', 'Failed' FROM capabilities
        WHERE capability_steps ~ E'^prepare-and-run-workflow (.*)\nawait-workflow$';

        INSERT INTO capability_state_actions (transition_id, action_type, arguments)
        SELECT transition_id, 'QueueWorkflow', (regexp_match(capability_steps, E'^prepare-and-run-workflow (.*)\nawait-workflow$'))[1] as arguments
        FROM capability_state_transitions cst
        JOIN capabilities c on cst.capability_name = c.capability_name
        WHERE cst.from_state = 'Start' and cst.to_state = 'Executing';
        INSERT INTO capability_state_actions (transition_id, action_type, arguments)
        SELECT transition_id, 'SendNotification', 'workflow-complete'
        FROM capability_state_transitions cst
        WHERE cst.from_state = 'Executing' and cst.to_state = 'Complete';
        INSERT INTO capability_state_actions (transition_id, action_type, arguments)
        SELECT transition_id, 'SendNotification', 'workflow-failed'
        FROM capability_state_transitions cst
        WHERE cst.from_state = 'Executing' and cst.to_state = 'Failed';
        """
    )
